
### load necessary R packages
library (Hmisc)
library (gplots)
library (plyr)
library (fda)

### load data - choose one of these possibilities below or define new line with own file
infil<-"temperature_changes_BI_DJF"
#infil<-"precipitation_changes_EA_JJA"

load(paste(infil,".RData",sep=""))

### choose season - line 13 or 14, or define new line with the choice for your own data
#level <- "JJA" # in case of "precipitation_changes_EA.RData"
level<-"DJF" # in case of "temperature_changes_BI.RData"

### some preparatory calculations
n<-length(levels(unique(tabulka$model)))  # number of the curves (RCM-GCM combinations)
curves <- matrix(nrow = n, ncol = 98)
for (m in 1:n){
  curves[m,]<-subset(tabulka$values, tabulka$model == unique(tabulka$model)[m])
}


###############################################################################################
###############################################################################################
### plot underlying data - like Fig. 1/2(a)
p0 <- ggplot(data = subset(tabulka, season == level), aes(x = year, y = values, colour = GCM, group = model, linetype = RCM))
p0 <- p0 + labs(x = "Years", y = "Value")
p0 <- p0 + geom_line(size = 1)
p0 <- p0 + theme_bw () 
p0 <- p0 + theme(strip.text = element_text(size = 12, face = "bold"))
print(p0)


###############################################################################################
###############################################################################################
### creation of smoothed curves for plotting - functional data - like shown in Fig. 1/2(b)

knots <- seq(1, 98, length.out = 20)  # definition of knots
range1 <- range(knots)
norder <- 4   # order of B-splines
nbasis <- length(knots) + norder - 2  # the size of the basis
bbasis <- create.bspline.basis(range1,nbasis,norder,knots)  # the basis of the B-splines
curv.Lfd <- int2Lfd(2)  # "Lfd" objects
lambda <- 1e2 # smoothing parameter lambda
curv.fdPar <- fdPar(bbasis,curv.Lfd,lambda) # "fdPar" object

## choice of lambda
  lambdas <- 10^seq(-4,2,by = 0.5)
  mean.gcv <- rep(0,length(lambdas)) 
  for(k in 1:length(lambdas)){
    curv.fdPari <- curv.fdPar
    curv.fdPari$lambda <- lambdas[k]
    tempSmoothi <- smooth.basis(1:98, t(curves), curv.fdPari)
    mean.gcv[k] <- mean(tempSmoothi$gcv)
  }
  best <- which.min(mean.gcv)
  lambdabest <- lambdas[best]
  
curv.fdPar$lambda <- lambdabest
tempSmooth <- smooth.basis(1:98, t(curves), curv.fdPar)
fdobjSmooth <- tempSmooth$fd
fdobjSmootheval <- eval.fd(fdobj = fdobjSmooth, evalarg = 1:98)
curves.smooth <- t(fdobjSmootheval)
df1 <- data.frame(year = rep(1986:2083, times = nrow(curves.smooth)), values = c(c(t(curves.smooth))), 
                  season = rep(level, each = 98*nrow(curves.smooth)), model = rep(unique(tabulka$model), each = 98, times = 1))
table_sm <- adply (df1, 1, transform,
                     GCM = substr (model, start = 6, stop = 11),
                     RCM = substr (model, start = 1, stop = 4))

### plot smoothed curves like in Fig. 1/2(b)
p0 <- ggplot(data = subset(table_sm, season == level), aes(x = year, y = values, colour = GCM, group = model, linetype = RCM))
p0 <- p0 + labs(x = "Years", y = "Value")
p0 <- p0 + geom_line(size = 1)
p0 <- p0 + theme_bw () + theme(strip.text = element_text(size = 12, face = "bold"))
print(p0)


###############################################################################################
###############################################################################################
#####  Prepare data for heatmaps like in Fig. 4 and 5 and for layout graphs 
## choice of the semimetric
## choose the qq<-0 for d0 and qq<-1 for d1 semimetric
#qq <- 0  
qq <- 1

source("npfda.R") # the file containting the definition of some of the functions used in the following code.
normalize <- function (x) {           # Definition of the scaling function - scaling of the distances to the interval [0;1]
  min <- min (x, na.rm = FALSE)
  range <- max (x, na.rm = FALSE) - min
  return ((x - min) / range)
}

distances <- semimetric.deriv (curves, curves,q = qq, 
                               range.grid = c(1,98), nknot = min (c(98-qq-4)%/%2, 20)) ##  distances calculation
colnames(distances)<-unique(tabulka$model)
rownames(distances)<-unique(tabulka$model)

a<-data.frame(dista = c(distances), dista.scaled = normalize (c(distances)), 
              model1 = colnames (distances), 
              model2 = rep (colnames (distances),each = nrow (distances)))    ## transformation in a different form for plotting


#### Create heatmaps like in Fig. 4 and 5

ll <- levels (a$model1)
llsor <- c(ll[c(4,13,1,3,5,14,6,15,7,16,8,17,9,18,10,19,2,11,20,22,12,21)])   ## define the ordering of the simulations
a$model1 <- factor(a$model1, levels = llsor)
a$model2 <- factor(a$model2, levels = llsor)

## plot the heatmap with normalized distances
p0 <- ggplot (data = a, 
              aes (x = model1, y = model2, fill = dista.scaled))
p0 <- p0 + geom_tile (color = "black") 
p0 <- p0 + scale_fill_gradientn (colours = heat.colors (100), name="Distances")
p0 <- p0 + coord_fixed (ratio = 1)  
p0 <- p0 + theme_bw ()
p0 <- p0 + theme (axis.text.x = element_text (angle = 90, vjust = 0.5), axis.title.x = element_blank(), axis.title.y = element_blank(), 
                  panel.grid.major = element_blank(), panel.grid.minor = element_blank(), legend.position="top",
                  panel.background = element_rect (fill = '#999999', colour = '#000000'), panel.spacing = unit (1, "lines"))
print (p0)


##### Prepare the input for the layout graphs 
##### resulting file must be processed by prepare_graphs.py and then manually treated using Gephi software
output_table<- 1-acast(a, model1 ~ model2, value.var = "dista.scaled")

ofile <- paste(infil, "_d",qq,".csv", sep = "")
write.table(output_table, ofile, sep = ";") 


#### Heatmap with dendrogram - like in example in Fig. 6
heatmap.2(distances, density.info = "none", trace = "none", 
          dendrogram = "row", key = FALSE, sepcolor="black",
          colsep=1:ncol(distances), sepwidth=c(0.001,0.001),
          rowsep=1:nrow(distances),
          margins = c(10,10))



